@@ -1,17 +1,21 @@
 """
-Machine learning module for Python
-==================================
-
-sklearn is a Python module integrating classical machine
-learning algorithms in the tightly-knit world of scientific Python
-packages (numpy, scipy, matplotlib).
-
-It aims to provide simple and efficient solutions to learning problems
-that are accessible to everybody and reusable in various contexts:
-machine-learning as a versatile tool for science and engineering.
-
-See http://scikit-learn.org for complete documentation.
+The :mod:`sklearn` module includes functions to configure global settings and
+get information about the working environment.
 """
+
+# Machine learning module for Python
+# ==================================
+#
+# sklearn is a Python module integrating classical machine
+# learning algorithms in the tightly-knit world of scientific Python
+# packages (numpy, scipy, matplotlib).
+#
+# It aims to provide simple and efficient solutions to learning problems
+# that are accessible to everybody and reusable in various contexts:
+# machine-learning as a versatile tool for science and engineering.
+#
+# See http://scikit-learn.org for complete documentation.
+
 import logging
 import os
 import random
@@ -38,7 +42,7 @@ logger = logging.getLogger(__name__)
 # Dev branch marker is: 'X.Y.dev' or 'X.Y.devN' where N is an integer.
 # 'X.Y.dev0' is the canonical version of 'X.Y.dev'
 #
-__version__ = "1.3.1"
+__version__ = "1.4.0"
 
 
 # On OSX, we can get a runtime error due to multiple OpenMP libraries loaded
