@@ -53,13 +53,6 @@ def test_min_dependencies_readme():
                 assert version == min_version, f"{package} has a mismatched version"
 
 
-@pytest.mark.skip(
-    reason=(
-        "For the release purpose, the upper limit of Cython is set to 3.0 since it "
-        "leads to performance regression. Scikit-learn can still be built with "
-        "Cython 3."
-    )
-)
 def test_min_dependencies_pyproject_toml():
     """Check versions in pyproject.toml is consistent with _min_dependencies."""
     # tomllib is available in Python 3.11
@@ -81,12 +74,15 @@ def test_min_dependencies_pyproject_toml():
     pyproject_build_min_versions = {}
     for requirement in build_requirements:
         if ">=" in requirement:
+            # Don't check NumPy: this requirement is only build time.
+            if "numpy>=1.25" in requirement:
+                continue
             package, version = requirement.split(">=")
             package = package.lower()
             pyproject_build_min_versions[package] = version
 
     # Only scipy and cython are listed in pyproject.toml
-    # NumPy is more complex using oldest-supported-numpy.
+    # NumPy is more complex using oldest-supported-numpy or >=1.25.
     assert set(["scipy", "cython"]) == set(pyproject_build_min_versions)
 
     for package, version in pyproject_build_min_versions.items():
